%TRANSHEATFLUX.M
%Computes transient heat flux statistics from ERA5 derived reanalysis data
%Requires file era5.trans.2010-2020.an.nc
%Brian Tang, 4/30/09
%Last modified by Talia Tamarin-Brodsky 04/15/24

clear all;
load coasts.mat

%GLOBAL VARIABLES AND CONSTANTS (DO NOT ALTER!)
global nctbx_options
nctbx_options.theAutoscale = 1;
nctbx_options.theAutoNaN = 1;
earthrad = 6.37E6; %m
cp = 1005.7; %J/(kg K)
g = 9.81; %m/s^2
Lv= 2.25*10^6; % J/kg/K

%USER DEFINED OPTIONS
%1: TRANSIENT HEAT FLUX AT A SINGLE LEVEL IN K m/s
%2: VERTICALLY AVERAGED TRANSIENT HEAT FLUX IN K m/s
%3: ZONALLY AVERAGED TRANSIENT HEAT FLUX IN K m/s
%4: ZONALLY AND VERTICALLY INTEGRATED TRANSIENT (INTERNAL) ENERGY TRANSPORT IN PW
opt = 1;
month = [1 12]; %Beginning and End months for averaging period


    %filein = 'vt.mon.mean.3_10.nc';
    filein = 'era5.trans.2010-2020.an.nc';
    lat = ncread(filein,'lat');
    lon = ncread(filein,'lon');
    p = double(ncread(filein,'level'));

    ve = ncread(filein,'ve'); 
    vq = ncread(filein,'vq');

%RESHAPE MATRIX DIMENDIONS
vt=permute(ve,[3 2 1]);   % the thermal heat flux v'theta'
vm=permute(vq,[3 2 1]);   % the moisture heat flux v'q'
%DATA DIMENSION ('level', 'lat', 'lon'); %% 

%ARRAY FOR THICKNESS OF INDIVIDUAL LAYERS USED FOR AVERAGING AND INTEGRATION
phalf = (p(1:end-1)+p(2:end))/2;
dp = [1012-phalf(1); phalf(1:end-1)-phalf(2:end); phalf(end)-50];

cint = 20; %number of contours for plotting;
%PLOT DATA

if (opt==1)
   disp('TRANSIENT HEAT FLUX AT A SINGLE LEVEL');
   disp(['Available levels to plot are: ', num2str(p')]);
   plotlev = input('Select level at which to plot: ');
   while(isempty(find(p==plotlev)))
      disp('Unavailable level chosen.');
      plotlev = input('Select level at which to plot: ');
   end
   
   figure;
   vtplev = squeeze(vt(find(p==plotlev),:,:));
   [c,h] = contourf(lon,lat,vtplev,cint);
   hold on; set(h,'LineStyle','none'); 
   colormap('jet')
   hold on
   plot(coastlines(1,:)+360,coastlines(2,:),'k','LineWidth',2);
   plot(coastlines(1,:),coastlines(2,:),'k','LineWidth',2);
   xlim([0 360])
   grid on;
   xlabel('Longitude','fontsize',14);
   ylabel('Latitude','fontsize',14);
   title(['Transient Heat Flux [K m/s] @ ',num2str(plotlev),'mb, Months: ',num2str(month(1)),'-',num2str(month(2))],'fontsize',14);
   set(gca,'fontsize',14,'layer','top');
   hold off;
   colorbar
   clim([-30 30])
elseif (opt==2)
   disp('VERTICALLY AVERAGED TRANSIENT HEAT FLUX');

   figure;
   vtpave = squeeze(sum(vt.*repmat(dp,[1 length(lat) length(lon)]),1)./sum(repmat(dp,[1 length(lat) length(lon)]),1));
   [c,h] = contourf(lon,lat,vtpave,cint);
   hold on; set(h,'LineStyle','none'); 
   colormap('jet')
   hold on
   plot(coastlines(1,:)+360,coastlines(2,:),'k','LineWidth',2);
   plot(coastlines(1,:),coastlines(2,:),'k','LineWidth',2);
   xlim([0 360])
   grid on;
   xlabel('Longitude','fontsize',14);
   ylabel('Latitude','fontsize',14);
   title(['Vertically Averaged Transient Heat Flux [K m/s], Months: ',num2str(month(1)),'-',num2str(month(2))],'fontsize',14);
   set(gca,'fontsize',14,'layer','top');
   hold off; 
   colorbar
   clim([-23 23])
elseif (opt==3)
   disp('ZONALLY AVERAGED TRANSIENT HEAT FLUX');
   %***********************************************
   %Compute zonally averaged transient heat flux in K m/s.
   %Variables you will need: vt('level', 'lat', 'lon') [K m/s] - transient heat flux
   figure;
   disp('Zonally AVERAGED TRANSIENT HEAT FLUX');

   %vtzave=; % compute here the zonal mean heat flux 

   [c,h] = contourf(lat,p,vtzave,cint);
   hold on; set(h,'LineStyle','none'); 
   colormap('jet')
   grid on;
   axis ij
   ylabel('Pressure','fontsize',14);
   xlabel('Latitude','fontsize',14);
   title(['Zonally Averaged Transient Heat Flux [K m/s], Months: ',num2str(month(1)),'-',num2str(month(2))],'fontsize',14);
   set(gca,'fontsize',14,'layer','top');
   colorbar
   clim([-23 23])
   hold off; 
   %***********************************************
elseif (opt==4)
   disp('ZONALLY AND VERTICALLY INTEGRATED TRANSIENT ENERGY FLUX');
   %***********************************************
   %Compute vertically and zonally integrated transient energy flux in PW (thermal plus latent heating).
   %Variables you will need: 
   %                vt('level', 'lat', 'lon') [K m/s] - transient heat flux
   %                vq('level', 'lat', 'lon') [kg/kg m/s] - transient moisture (specific humidity) flux
   %				cp [J/(kg K] - specific heat of dry air
   %                Lv [J/kg/K]- latent heat of vaporization
   %				g [m/s^2] - gravity
   %				earthrad [m] - Earth's radius
   %				lat [deg] - latitude (note: here in degrees!)
   %				dp [hPa] - thickness of layers integrating over (convert to Pa!!)

   %vh=;  %compute the total transient energy flux v'h'

   figure;
   %vhzave=;  %take the zonal mean of the toal transient energy flux
   %vhpzave=; %calculate the vertical integral of that zonal mean
   %H = ; %calculate the total zonally and vetically integrated energy flux
   
   %***********************************************
   plot(H./1E15,lat,'linewidth',2);
   grid on;
   xlabel('Transient Energy Flux [PW]','fontsize',14);
   ylabel('Latitude','fontsize',14);
   title(['Transient Energy Flux [PW], Months: ',num2str(month(1)),'-',num2str(month(2))],'fontsize',14);
   set(gca,'fontsize',14,'layer','top');
end
