############################################################
## Computes transient heat flux statistics from ERA5 derived reanalysis data
## Requires file era5.trans.2010-2020.an.nc
## Funing Li, 4/15/2024 
############################################################

## Import packages
## Try "pip install xx" if a package installation is needed
import numpy as np 
import netCDF4    
import matplotlib 
import matplotlib.pyplot as plt 
import cartopy.crs as ccrs  

## CONSTANTS
earthrad = 6.37e6  # m
cp       = 1005.7  # J/(kg K)
g        = 9.81    # m/s^2
lv       = 2.25e6  # J/kg/K

##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
## USER DEFINED OPTIONS
## 1: TRANSIENT HEAT FLUX AT A SINGLE LEVEL IN K m/s
## 2: VERTICALLY AVERAGED TRANSIENT HEAT FLUX IN K m/s
## 3: ZONALLY AVERAGED TRANSIENT HEAT FLUX IN K m/s
## 4: ZONALLY AND VERTICALLY INTEGRATED TRANSIENT (INTERNAL) ENERGY TRANSPORT IN PW
opt      = 1;
##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

## READ DATA  
## Alternatively, one can use "xarray" to read .nc data:
##               import xarray as xr
##               f  = xr.open_dataset('era5.trans.2010-2020.an.nc')
##               ve = f['ve']  
f                = netCDF4.Dataset('era5.trans.2010-2020.an.nc')
vt               = np.asarray(f.variables['ve'])    # thermal heat flux  v'theta'; dim: [level,lat,lon]
vm               = np.asarray(f.variables['vq'])    # moisture heat flux v'q'    ; dim: [level,lat,lon] 
lat              = np.asarray(f.variables['lat'])   # dim: [lat]    
lon              = np.asarray(f.variables['lon'])   # dim: [lon]  
level            = np.asarray(f.variables['level']) # dim: [level]   
lon2d,lat2d      = np.meshgrid(lon,lat)  

## ARRAY FOR THICKNESS OF INDIVIDUAL LAYERS USED FOR AVERAGING AND INTEGRATION
phalf        = (level[0:-1]+level[1:])/2 
dp           = np.zeros(len(level))
dp[0]        = 1012-phalf[0]
dp[1:-1]     = phalf[0:-1]-phalf[1:]
dp[-1]       = phalf[-1]-50

## PLOT DATA
print('----------   opt = %i   ----------'%(opt))
if (opt==1): 
    print('TRANSIENT HEAT FLUX AT A SINGLE LEVEL')
    print('Available levels to plot are: ', level[:], 'mb')
    print('!!!NOTE: changing "plotlev" to other levels to plot for other levels')
    ## prepare data
    plotlev   = 850 
    iplev     = np.where(level==plotlev)[0]
    vtplev    = np.squeeze(vt[iplev])
    ## make a plot
    fontsize  = 4
    fig       = plt.figure(figsize=[4.5, 2],constrained_layout=False,dpi=300) 
    ax        = fig.add_subplot(projection=ccrs.PlateCarree(central_longitude=180))   
    bounds    = np.arange(-32,33,4)
    cmap      = plt.colormaps['jet'] 
    cl        = ax.contourf(lon2d, lat2d, vtplev,  bounds, transform= ccrs.PlateCarree(),cmap=cmap,extend='both') 
    ax.coastlines(linewidth=0.2,color = 'k') 
    ax.set_extent([-180,180,-90,90], crs=ccrs.PlateCarree()) 
    gl = ax.gridlines(draw_labels=True,linewidth=.2, color='gray', alpha=0.5, linestyle='-.')
    gl.xlabel_style = {"size" : fontsize};        gl.ylabel_style = {"size" : fontsize}
    gl.top_labels    = False;                     gl.right_labels  = False
    gl.bottom_labels = True;                      gl.left_labels   = True  
    cb=fig.colorbar(cl,ax=[ax],ticks=bounds[1:-1:2],shrink = 0.9,aspect=70,orientation='vertical',pad=0.015,fraction=0.2);
    cb.ax.tick_params(labelsize=fontsize,width=0.2); cb.outline.set_linewidth(0.2)
    ax.set_title('Annual Transient Heat Flux [K m/s] @ %i mb'%(plotlev),fontsize=fontsize);
elif (opt==2): 
    print('VERTICALLY AVERAGED TRANSIENT HEAT FLUX')
    ## prepare data
    vtpave = np.sum(np.multiply(vt, dp[:, None, None]),axis=0)/np.sum(dp) 
    ## make a plot
    fontsize  = 4
    fig       = plt.figure(figsize=[4.5, 2],constrained_layout=False,dpi=300) 
    ax        = fig.add_subplot(projection=ccrs.PlateCarree(central_longitude=180))   
    bounds    = np.arange(-24,25,3)
    cmap      = plt.colormaps['jet'] 
    cl        = ax.contourf(lon2d, lat2d, vtpave,  bounds, transform= ccrs.PlateCarree(),cmap=cmap,extend='both') 
    ax.coastlines(linewidth=0.2,color = 'k')  
    ax.set_extent([-180,180,-90,90], crs=ccrs.PlateCarree()) 
    gl = ax.gridlines(draw_labels=True,linewidth=.2, color='gray', alpha=0.5, linestyle='-.')
    gl.xlabel_style = {"size" : fontsize};        gl.ylabel_style = {"size" : fontsize}
    gl.top_labels    = False;                     gl.right_labels  = False
    gl.bottom_labels = True;                      gl.left_labels   = True  
    cb=fig.colorbar(cl,ax=[ax],ticks=bounds[1:-1:2],shrink = 0.9,aspect=70,orientation='vertical',pad=0.015,fraction=0.2);
    cb.ax.tick_params(labelsize=fontsize,width=0.2); cb.outline.set_linewidth(0.2)
    ax.set_title('Annual Vertically Averaged Transient Heat Flux [K m/s]',fontsize=fontsize) 
elif (opt==3):
    print('ZONALLY AVERAGED TRANSIENT HEAT FLUX')
    ## prepare data
    ##     Compute zonally averaged transient heat flux in K m/s.
    ##     Variables you will need: vt('level', 'lat', 'lon') [K m/s] - transient heat flux  
    # vtpave    =   ## compute here the zonal mean heat flux
    ## make a plot
    fontsize  = 5 
    fig       = plt.figure(figsize=[4., 2],constrained_layout=False,dpi=300) 
    ax        = fig.add_subplot()  
    x, y      = np.meshgrid(lat,level) 
    bounds    = np.arange(-21,23,3)
    cmap      = plt.colormaps['jet'] 
    cl        = ax.contourf(x, y, vtpave, bounds, cmap=cmap,extend='both') 
    ax.set_xlabel("Latitude", fontsize=fontsize)
    ax.set_ylabel("Pressure [hPa]", fontsize=fontsize)
    ax.tick_params(labelsize=fontsize)
    ax.set_ylim(1000,100)
    cb=fig.colorbar(cl,ax=[ax],ticks=bounds[1:-1:2],shrink = 1.,aspect=60,orientation='vertical',pad=0.015,fraction=0.2);
    cb.ax.tick_params(labelsize=fontsize,width=0.2); cb.outline.set_linewidth(0.2)
    ax.set_title('Annual Zonally Averaged Transient Heat Flux [K m/s]',fontsize=fontsize) 
elif (opt==4):
    print('ZONALLY AND VERTICALLY INTEGRATED TRANSIENT HEAT (ENERGY) FLUX')
    ## prepare data
    ##     Compute vertically and zonally integrated transient energy flux in PW (thermal plus latent heating).
    ##     Variables you will need: 
    ##              vt('level', 'lat', 'lon') [K m/s] - transient heat flux
    ##              vq('level', 'lat', 'lon') [kg/kg m/s] - transient moisture (specific humidity) flux
    ##				cp [J/(kg K) - specific heat of dry air
    ##              Lv [J/kg/K]- latent heat of vaporization
    ##				g [m/s^2] - gravity
    ##				earthrad [m] - Earth's radius
    ##				lat [deg] - latitude (here in degrees!)
    ##				dp [hPa] - thickness of layers integrating over (convert to Pa!!)  
    # vh        =      ## %compute the total transient energy flux v'h'
    # vhzave    =      ## take the zonal mean of the toal transient energy flux
    # vhpzave   =      ## calculate the vertical integral of that zonal mean
    # H         =      ## calculate the total zonally and vetically integrated energy flux
    ## make a plot
    fontsize  = 6 
    fig       = plt.figure(figsize=[3.5, 2],constrained_layout=False,dpi=300) 
    ax        = fig.add_subplot()  
    ax.plot(H/(1e15), lat, color='k',linewidth=1.)
    ax.set_xlabel("Transient Energy Flux [PW]", fontsize=fontsize)
    ax.set_ylabel("Latitude", fontsize=fontsize)
    ax.tick_params(labelsize=fontsize) 
    ax.set_title('Transient Energy Flux [PW]',fontsize=fontsize) 
#
#   
#
## SAVE PLOT
figfile_out = 'fig_option%i_era5.png'%(opt)   
plt.savefig(figfile_out, bbox_inches='tight')
#
#
#